@@ -206,7 +206,8 @@ int init_crypto(struct crypt_device *ctx)
 		log_err(ctx, _("Cannot initialize crypto backend.\n"));
 
 	if (!r && !_crypto_logged) {
-		log_dbg("Crypto backend (%s) initialized.", crypt_backend_version());
+		log_dbg("Crypto backend (%s) initialized in cryptsetup library version %s.",
+			crypt_backend_version(), PACKAGE_VERSION);
 		if (!uname(&uts))
 			log_dbg("Detected kernel %s %s %s.",
 				uts.sysname, uts.release, uts.machine);
@@ -1387,6 +1388,14 @@ int crypt_resize(struct crypt_device *cd, const char *name, uint64_t new_size)
 		goto out;
 	}
 
+	if (crypt_loop_device(crypt_get_device_name(cd))) {
+		log_dbg("Trying to resize underlying loop device %s.",
+			crypt_get_device_name(cd));
+		/* Here we always use default size not new_size */
+		if (crypt_loop_resize(crypt_get_device_name(cd)))
+			log_err(NULL, _("Cannot resize loop device.\n"));
+	}
+
 	r = device_block_adjust(cd, dmd.data_device, DEV_OK,
 				dmd.u.crypt.offset, &new_size, &dmd.flags);
 	if (r)
